CREATE TABLE IF NOT EXISTS `ScheduledEvents`
(
    `Id`             BIGINT             NOT NULL AUTO_INCREMENT,
    `Interval`    ENUM ('daily', 'weekly', 'monthly', 'yearly') NOT NULL,
    `Name`           VARCHAR(255)       NOT NULL,
    `Description` VARCHAR(255)                                  NULL,
    `ChannelId`      BIGINT             NULL,
    `StartTime`      DATETIME(6)        NOT NULL,
    `EndTime`        DATETIME(6)        NULL,
    `EntityType`     ENUM ('1','2','3') NOT NULL,
    `Location`       VARCHAR(255)       NULL,
    `ServerId`       BIGINT,
    `CreatedAt`      DATETIME(6)        NULL DEFAULT CURRENT_TIMESTAMP(6),
    `LastModifiedAt` DATETIME(6)        NULL DEFAULT CURRENT_TIMESTAMP(6) ON UPDATE CURRENT_TIMESTAMP(6),
    PRIMARY KEY (`Id`),
    FOREIGN KEY (`ServerId`) REFERENCES `Servers` (`ServerId`)
);

CREATE TABLE IF NOT EXISTS `ScheduledEventsHistory`
(
    `Id`          BIGINT(20)         NOT NULL,
    `Interval`    ENUM ('daily', 'weekly', 'monthly', 'yearly') NOT NULL,
    `Name`        VARCHAR(255)       NOT NULL,
    `Description` VARCHAR(255)                                  NULL,
    `ChannelId`   BIGINT             NULL,
    `StartTime`   DATETIME(6)        NOT NULL,
    `EndTime`     DATETIME(6)        NULL,
    `EntityType`  ENUM ('1','2','3') NOT NULL,
    `Location`    VARCHAR(255)       NULL,
    `ServerId`    BIGINT(20) DEFAULT NULL,
    `Deleted`     BOOL       DEFAULT FALSE,
    `DateFrom`    DATETIME(6)        NOT NULL,
    `DateTo`      DATETIME(6)        NOT NULL
);

DROP TRIGGER IF EXISTS `TR_ScheduledEventsUpdate`;

CREATE TRIGGER `TR_ScheduledEventsUpdate`
    AFTER UPDATE
    ON `ScheduledEvents`
    FOR EACH ROW
BEGIN
    INSERT INTO `ScheduledEventsHistory` (`Id`,
                                          `Interval`,
                                          `Name`,
                                          `Description`,
                                          `ChannelId`,
                                          `StartTime`,
                                          `EndTime`,
                                          `EntityType`,
                                          `Location`,
                                          `ServerId`,
                                          `DateFrom`,
                                          `DateTo`)
    VALUES (OLD.Id, OLD.Interval, OLD.Name, OLD.Description, OLD.ChannelId, OLD.StartTime, OLD.EndTime, OLD.EntityType,
            OLD.Location, OLD.ServerId, OLD.LastModifiedAt,
            CURRENT_TIMESTAMP(6));
END;

DROP TRIGGER IF EXISTS `TR_ScheduledEventsDelete`;

CREATE TRIGGER `TR_ScheduledEventsDelete`
    AFTER DELETE
    ON `ScheduledEvents`
    FOR EACH ROW
BEGIN
    INSERT INTO `ScheduledEventsHistory` (`Id`,
                                          `Interval`,
                                          `Name`,
                                          `Description`,
                                          `ChannelId`,
                                          `StartTime`,
                                          `EndTime`,
                                          `EntityType`,
                                          `Location`,
                                          `ServerId`,
                                          `Deleted`,
                                          `DateFrom`,
                                          `DateTo`)
    VALUES (OLD.Id, OLD.Interval, OLD.Name, OLD.Description, OLD.ChannelId, OLD.StartTime, OLD.EndTime, OLD.EntityType,
            OLD.Location, OLD.ServerId, TRUE, OLD.LastModifiedAt,
            CURRENT_TIMESTAMP(6));
END;