from cpl_core.configuration import ConfigurationABC
from cpl_core.database.context import DatabaseContextABC
from cpl_discord.service import DiscordBotServiceABC
from cpl_translation import TranslatePipe
from discord.ext import tasks

from bot_core.abc.task_abc import TaskABC
from bot_core.logging.task_logger import TaskLogger
from bot_core.service.message_service import MessageService
from bot_data.abc.server_repository_abc import ServerRepositoryABC
from modules.base.service.event_service import EventService


class ScheduledEventsWatcher(TaskABC):
    def __init__(
        self,
        config: ConfigurationABC,
        logger: TaskLogger,
        bot: DiscordBotServiceABC,
        db: DatabaseContextABC,
        servers: ServerRepositoryABC,
        events: EventService,
        message_service: MessageService,
        t: TranslatePipe,
    ):
        TaskABC.__init__(self)

        self._config = config
        self._logger = logger
        self._bot = bot
        self._db = db
        self._servers = servers
        self._events = events
        self._message_service = message_service
        self._t = t

        if not self._is_maintenance():
            self.watch.start()

    @tasks.loop(hours=12)
    async def watch(self):
        self._logger.info(__name__, "Watching scheduled events")
        try:
            for guild in self._bot.guilds:
                await self._events.check_and_create_scheduled_events(guild)
        except Exception as e:
            self._logger.error(__name__, f"Watching scheduled events failed", e)

    @watch.before_loop
    async def wait(self):
        await self._wait_until_ready()
